/* Copyright (C) 2006 Damir Zucic */

/*=============================================================================

			    hyphob_function2.c

Purpose:
	Draw the hydrophobicity function F2.  The sequence stored to the
	main sequence buffer  is used  to calculate  the function value.

Input:
	(1) Pointer to the storage where the minimal function value will
	    be stored.
	(2) Pointer to the storage where the maximal function value will
	    be stored.
	(3) Pointer to RuntimeS structure.

Output:
	(1) Function F2 calculated and stored.
	(2) Return value.

Return value:
	(1) Positive on success.
	(2) Negative on failure.

Notes:
	(1) The function  F2 may be modified and used for many purposes.
	    Originally, it was introduced while searching for the method
	    which will be suitable for prediction of the porin secondary
	    structure.

========includes:============================================================*/

#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "defines.h"
#include "typedefs.h"

/*======calculate the hydrophobicity function F2:============================*/

int HyphobFunction2_ (double *min_functionP, double *max_functionP,
		      RuntimeS *runtimeSP)
{
int             residuesN, residueI;   /* Do not use size_t instead of int ! */
int		max_length;
int		count1, count3;
int		windowI, combinedI;
char		*residue_nameP;
char		residue_nameA[RESNAMESIZE];
double		function_value;

/*------prepare some parameters:---------------------------------------------*/

/* The number of residues in sequence buffer: */
residuesN = (int) runtimeSP->residuesN;
if (residuesN == 0) return -1;

/* The maximal residue name length: */
max_length = RESNAMESIZE - 1;

/*------calculate the function F2:-------------------------------------------*/

/* Initialize the extreme values: */
*min_functionP = +999999.0;
*max_functionP = -999999.0;

/* 20060404.1028: */
/* Scan a window of  21 residue.  Divide this window into three subwindows. */
/* The first and the third subwindow consist of 9 residues  and the central */ 
/* window has 3 residues.  Ignore the central window and find the number of */
/* LEU, ILE, VAL, MET, PHE, TYR, TRP  in the first and the third subwindow. */
/* Check which count is larger.  The function value is equal to this count. */

/* Scan the sequence: */
for (residueI = 0; residueI < residuesN; residueI++)
	{
	/* Reset the function value, it might be initialized before: */
	*(runtimeSP->function2P + residueI) = 0.0;

	/* Reset the residue counts for the first and the third subwindow: */
	count1 = 0;
	count3 = 0;

	/* Scan the sliding window: */
	for (windowI = -10; windowI < 10; windowI++)
		{
		/* Prepare and check the combined index: */
		combinedI = residueI + windowI;
		if (combinedI < 0) continue;
		if (combinedI >= residuesN) continue;

		/* Pointer  to  the residue  name of */
		/* the residue defined by combinedI: */
		residue_nameP = runtimeSP->sequenceP + combinedI * max_length;

		/* Copy the residue name: */
		strncpy (residue_nameA, residue_nameP, max_length);
		residue_nameA[max_length] = '\0';

		/* Check is this residue from the set of seven selected */
		/* hyphob. residues. If it is, check to which subwindow */
		/* this residue belongs.  If it belongs to the first or */
		/* the third subwindow, increment the respective count. */
		if ((strcmp (residue_nameA, "LEU") == 0) ||
		    (strcmp (residue_nameA, "ILE") == 0) ||
		    (strcmp (residue_nameA, "VAL") == 0) ||
		    (strcmp (residue_nameA, "MET") == 0) ||
		    (strcmp (residue_nameA, "PHE") == 0) ||
		    (strcmp (residue_nameA, "TYR") == 0) ||
		    (strcmp (residue_nameA, "TRP") == 0))
			{
			if (windowI <= -2) count1++;
			else if (windowI >= 2) count3++;
			}
		}

	/* Check which count is larger and use it as a function value: */
	function_value = (double) count1;
	if (count3 > count1) function_value = (double) count3;

	/* Store the function value: */
	*(runtimeSP->function2P + residueI) = function_value;

	/* Find the extreme values: */
	if (function_value < *min_functionP) *min_functionP = function_value;
	if (function_value > *max_functionP) *max_functionP = function_value;

	/* End of residueI loop: */
	}

/*---------------------------------------------------------------------------*/

return 1;
}

/*===========================================================================*/


