/* Copyright (C) 2004-2006 Damir Zucic */

/*=============================================================================

			    hyphob_function3.c

Purpose:
	Draw the hydrophobicity function F3.  The sequence stored to the
	main sequence buffer  is used  to calculate  the function value.

Input:
	(1) Pointer to the storage where the minimal function value will
	    be stored.
	(2) Pointer to the storage where the maximal function value will
	    be stored.
	(3) Pointer to RuntimeS structure.

Output:
	(1) Function F3 calculated and stored.
	(2) Return value.

Return value:
	(1) Positive on success.
	(2) Negative on failure.

Notes:
	(1) The function  F3 may be modified and used for many purposes.
	    Originally, it was introduced while searching for the method
	    which will be suitable for prediction of the porin secondary
	    structure.

========includes:============================================================*/

#include <stdio.h>

#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "defines.h"
#include "typedefs.h"

/*======function prototypes:=================================================*/

int		HyphobFunction1_ (double *, double *, RuntimeS *);
int		HyphobFunction2_ (double *, double *, RuntimeS *);
void		InitHyphob_ (RuntimeS *);

/*======calculate the hydrophobicity function F3:============================*/

int HyphobFunction3_ (double *f3minP, double *f3maxP, RuntimeS *runtimeSP)
{
int		residuesN, residueI;   /* Do not use size_t instead of int ! */
int		max_length;
double		f1, f1min, f1max;
double		f2, f2min, f2max;
double		f3;

/*------prepare some parameters:---------------------------------------------*/

/* The number of residues in sequence buffer: */
residuesN = (int) runtimeSP->residuesN;
if (residuesN == 0) return -1;

/* The maximal residue name length: */
max_length = RESNAMESIZE - 1;

/*------initialize (reset) F3:-----------------------------------------------*/

/* Initialize F3: */
for (residueI = 0; residueI < residuesN; residueI++)
        {
        /* Reset the function F3, it might be initialized before: */
        *(runtimeSP->function3P + residueI) = 0.0;
        }

/*------calculate the function F1:-------------------------------------------*/

/* Calculate the function F1: */
HyphobFunction1_ (&f1min, &f1max, runtimeSP);

/*------calculate the function F4:-------------------------------------------*/

/* Calculate the function F2: */
HyphobFunction2_ (&f2min, &f2max, runtimeSP);

/*------calculate the function F3:-------------------------------------------*/

/* Initialize the extreme values: */
*f3minP = +999999.0;
*f3maxP = -999999.0;

/* 20060321.1458: */
/* The function F3 is calculated from F1 and F2. */

/* Scan the sequence and calculate F3: */
for (residueI = 0; residueI < residuesN; residueI++)
	{
	/* Calculate F3: */
        f1 = *(runtimeSP->function1P + residueI);
        f2 = *(runtimeSP->function2P + residueI);
	/* There should be  at least  three */
	/* charged or zwitterionic residues */
	/* in the window of  five residues: */
	f3 = f2 * f1;

	/* Store F3: */
	*(runtimeSP->function3P + residueI) = f3;
	}

/*------find the extreme values of F3:---------------------------------------*/

/* Find the extreme values of F3: */
for (residueI = 0; residueI < residuesN; residueI++)
	{
	/* Find extreme values for F3: */
	f3 = *(runtimeSP->function3P + residueI);
	if (f3 < *f3minP) *f3minP = f3;
	if (f3 > *f3maxP) *f3maxP = f3;
	}

/*---------------------------------------------------------------------------*/

return 1;
}

/*===========================================================================*/


