/* Copyright (C) 2004-2006 Damir Zucic */

/*=============================================================================

			    hyphob_function6.c

Purpose:
	Draw the hydrophobicity function F6.  The sequence stored to the
	main sequence buffer  is used  to calculate  the function value.

Input:
	(1) Pointer to the storage where the minimal function value will
	    be stored.
	(2) Pointer to the storage where the maximal function value will
	    be stored.
	(3) Pointer to RuntimeS structure.

Output:
	(1) Function F6 calculated and stored.
	(2) Return value.

Return value:
	(1) Positive on success.
	(2) Negative on failure.

Notes:
	(1) The function  F6 may be modified and used for many purposes.
	    Originally, it was introduced while searching for the method
	    which will be suitable for prediction of the porin secondary
	    structure.

========includes:============================================================*/

#include <stdio.h>

#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "defines.h"
#include "typedefs.h"

/*======function prototypes:=================================================*/

int		HyphobFunction1_ (double *, double *, RuntimeS *);
int		HyphobFunction2_ (double *, double *, RuntimeS *);
int		HyphobFunction3_ (double *, double *, RuntimeS *);
int		HyphobFunction4_ (double *, double *, RuntimeS *);
void		InitHyphob_ (RuntimeS *);

/*======calculate the hydrophobicity function F6:============================*/

int HyphobFunction6_ (double *f6minP, double *f6maxP, RuntimeS *runtimeSP)
{
int		residuesN, residueI;   /* Do not use size_t instead of int ! */
int		max_length;
int		old_scaleI;
double		f3, f3min, f3max;
double		f4, f4min, f4max;
double		f6;

/*------prepare some parameters:---------------------------------------------*/

/* The number of residues in sequence buffer: */
residuesN = (int) runtimeSP->residuesN;
if (residuesN == 0) return -1;

/* The maximal residue name length: */
max_length = RESNAMESIZE - 1;

/*------copy the old scale index:--------------------------------------------*/

/* Copy the old hydrophobicity scale index: */
old_scaleI = runtimeSP->hydrophobicity_scaleI;

/*------initialize (reset) F6:-----------------------------------------------*/

/* Initialize F6: */
for (residueI = 0; residueI < residuesN; residueI++)
        {
        /* Reset the function F6, it might be initialized before: */
        *(runtimeSP->function6P + residueI) = 0.0;
        }

/*------calculate the function F3:-------------------------------------------*/

/* Use the octanol scale (suitable for the transmembrane region): */
runtimeSP->hydrophobicity_scaleI = 0;

/* Update the hydrophobicity values in the sequence buffer: */
InitHyphob_ (runtimeSP);

/* Calculate the function F3: */
HyphobFunction3_ (&f3min, &f3max, runtimeSP);

/*------calculate the function F4:-------------------------------------------*/

/* Use the interface scale (suitable for the polar interface): */
runtimeSP->hydrophobicity_scaleI = 3;

/* Update the hydrophobicity values in the sequence buffer: */
InitHyphob_ (runtimeSP);

/* Calculate the function F4: */
HyphobFunction4_ (&f4min, &f4max, runtimeSP);

/*------restore the old hydrophobicity scale index:--------------------------*/

/* Restore the scale index: */
runtimeSP->hydrophobicity_scaleI = old_scaleI;

/* Update the hydrophobicity values in the sequence buffer: */ 
InitHyphob_ (runtimeSP);

/*------calculate the function F6:-------------------------------------------*/

/* Initialize the extreme values: */
*f6minP = +999999.0;
*f6maxP = -999999.0;

/* 20060221.1138: */
/* The function F6 is calculated from F3 and F4. */

/* Scan the sequence and calculate F6: */
for (residueI = 0; residueI < residuesN; residueI++)
	{
	/* Calculate F6: */
        f3 = *(runtimeSP->function3P + residueI);
        f4 = *(runtimeSP->function4P + residueI);
	/* Possible extreme values for octanol scale: min=-3.64, max=+2.09 */
	f6 = sqrt (((f4 + 3.64) / 5.63) * ((2.09 - f3) / 5.63));

	/* Store F6: */
	*(runtimeSP->function6P + residueI) = f6;
	}

/*------find the extreme values of F6:---------------------------------------*/

/* Find the extreme values of F6: */
for (residueI = 0; residueI < residuesN; residueI++)
	{
	/* Find extreme values for F6: */
	f6 = *(runtimeSP->function6P + residueI);
	if (f6 < *f6minP) *f6minP = f6;
	if (f6 > *f6maxP) *f6maxP = f6;
	}

/*---------------------------------------------------------------------------*/

return 1;
}

/*===========================================================================*/


