/* Copyright (C) 2006 Damir Zucic */

/*=============================================================================

			    hyphob_function7.c

Purpose:
	Draw the hydrophobicity function F7.  The sequence stored to the
	main sequence buffer  is used  to calculate  the function value.

Input:
	(1) Pointer to the storage where the minimal function value will
	    be stored.
	(2) Pointer to the storage where the maximal function value will
	    be stored.
	(3) Pointer to RuntimeS structure.

Output:
	(1) Function F7 calculated and stored.
	(2) Return value.

Return value:
	(1) Positive on success.
	(2) Negative on failure.

Notes:
	(1) The function  F7 may be modified and used for many purposes.
	    Originally, it was introduced while searching for the method
	    which will be suitable for prediction of the porin secondary
	    structure.

========includes:============================================================*/

#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "defines.h"
#include "typedefs.h"

/*======calculate the hydrophobicity function F7:============================*/

int HyphobFunction7_ (double *min_functionP, double *max_functionP,
		      RuntimeS *runtimeSP)
{
int             residuesN, residueI;   /* Do not use size_t instead of int ! */
int		max_length;
int		windowI, combinedI;
double		function_value;
char		*residue_nameP;
char		residue_nameA[RESNAMESIZE];
double		weighting_factor;
double		hydrophobicity;

/*------prepare some parameters:---------------------------------------------*/

/* The number of residues in sequence buffer: */
residuesN = (int) runtimeSP->residuesN;
if (residuesN == 0) return -1;

/* The maximal residue name length: */
max_length = RESNAMESIZE - 1;

/*------calculate the function F7:-------------------------------------------*/

/* Initialize the extreme values: */
*min_functionP = +999999.0;
*max_functionP = -999999.0;

/* 20060313.1816: */
/* Scan the window of 7 residues. Calculate the negative hydrophob. using */
/* only ASP, ASN, GLU, GLN, ARG, LYS, THR and SER. Ignore other residues. */

/* Scan the sequence: */
for (residueI = 0; residueI < residuesN; residueI++)
	{
	/* Reset the function value, it might be initialized before: */
	*(runtimeSP->function7P + residueI) = 0.0;

	/* Reset the local variable reserved for the function value: */
	function_value = 0.0;

	/* Scan the sliding window: */
	for (windowI = 0; windowI < 7; windowI++)
		{
		/* Prepare and check the combined index: */
		combinedI = residueI + windowI - 3;
		if (combinedI < 0) continue;
		if (combinedI >= residuesN) continue;

		/* Pointer  to  the residue  name of */
		/* the residue defined by combinedI: */
		residue_nameP = runtimeSP->sequenceP + combinedI * max_length;

		/* Copy the residue name: */
		strncpy (residue_nameA, residue_nameP, max_length);
		residue_nameA[max_length] = '\0';

		/* Prepare the weighting factor.  Use the factor  -1 for */
		/* selected polar residues and 0 for all other residues: */ 
		if ((strcmp (residue_nameA, "ASP") == 0) ||
		    (strcmp (residue_nameA, "ASN") == 0) ||
		    (strcmp (residue_nameA, "GLU") == 0) ||
		    (strcmp (residue_nameA, "GLN") == 0) ||
		    (strcmp (residue_nameA, "ARG") == 0) ||
		    (strcmp (residue_nameA, "LYS") == 0) ||
		    (strcmp (residue_nameA, "THR") == 0) ||
		    (strcmp (residue_nameA, "SER") == 0))
			{
			weighting_factor = -1.0;
			}
		else weighting_factor = 0.0;

		/* Prepare the hydrophobicity value: */
		hydrophobicity = *(runtimeSP->hydrophobicityP + combinedI);

		/* Update the function value: */
		function_value += weighting_factor * hydrophobicity;
		}

	/* Reduce the function value (arbitrary scaling): */
	function_value *= 0.1;

	/* Store the function value: */
	*(runtimeSP->function7P + residueI) = function_value;

	/* Find the extreme values: */
	if (function_value < *min_functionP) *min_functionP = function_value;
	if (function_value > *max_functionP) *max_functionP = function_value;

	/* End of residueI loop: */
	}

/*---------------------------------------------------------------------------*/

return 1;
}

/*===========================================================================*/


